
`define     JEIDA_MAPPING
`define     USE_OSC

`ifdef USE_OSC
//! MIPI: VFP/VSW/VBP = 8/4/4
//! PCLK : 490 / 3.5 = 140 MHZ
//! (129+40+40+1920) * (1080+8+4+4) * 60 ~= 140 MHz
`define     PATTERN_LEN     2
`define     PATTERN         2'b11
`define     START_X         0
`define     WIDTH           1920
`define     HEIGHT          1080
`define     VFP             8
`define     VSW             4
`define     VBP             4
`define     HFP             129
`define     HSW             40
`define     HBP             40
`define     VSYNC_DELAY     180
`endif

`define     HS_EN_DLY       'd4

module Dsi2Lvds_Top
(   input           RESET_N
,   input           OSC_50M
//! input
,   input           HS_CLK_P
,   input           HS_CLK_N
,   input           HS_DATA3_P
,   input           HS_DATA3_N
,   input           HS_DATA2_P
,   input           HS_DATA2_N
,   input           HS_DATA1_P
,   input           HS_DATA1_N
,   input           HS_DATA0_P
,   input           HS_DATA0_N
,   input  [1:0]    LP_DATA0
//! output
,   output          LVDS_CLK_P
,   output          LVDS_CLK_N
,   output          LVDS_D0_P
,   output          LVDS_D0_N
,   output          LVDS_D1_P
,   output          LVDS_D1_N
,   output          LVDS_D2_P
,   output          LVDS_D2_N
,   output          LVDS_D3_P
,   output          LVDS_D3_N
);
localparam  DT_RGB888 = 6'h3E;

//! byte_clk = bit_clk / 4 = 420 / 4 = 105MHz
wire            byte_clk    /* synthesis syn_keep = 1 */;
wire            byte_ready  ;
//wire            hs_en       ;
//wire [3:0]      term_en     ;

wire [7:0]      byte_d3, byte_d2, byte_d1, byte_d0;
wire [7:0]      dbf_d3 , dbf_d2 , dbf_d1 , dbf_d0 ;

wire            pixel_clk   /* synthesis syn_keep = 1 */;
wire            lvds_eclk   /* synthesis syn_keep = 1 */;
wire            lvds_pclk   ;
wire            sp_en       ;
wire            lp_en       ;
wire            lp_av_en    ;
wire            ecc_ok      ;
wire [15:0]     wc          ;
wire [ 5:0]     dt          ;
wire [3:0]      payload_dv  ;
wire [31:0]     payload     ;

wire            b2p_vsync   ;
wire            b2p_hsync   ;
wire            b2p_de      ;
wire [23:0]     b2p_pixel   ;

wire            ext_vsync   ;
wire            ext_hsync   ;
wire            ext_de      ;
wire [23:0]     ext_pixel   ;

wire [ 7:0]     lvds_r      ;
wire [ 7:0]     lvds_g      ;
wire [ 7:0]     lvds_b      ;
wire            lvds_vs     ;
wire            lvds_hs     ;
wire            lvds_de     ;


`ifdef SIM
wire VCC;
VCC VCC_cZ (
    .V(VCC)
);

GSR GSR (
    .GSRI(VCC) 
);
`endif


reg  [1:0]  lp_reg  [1:0];
reg         odt_en_msk  = 'b0;
reg         rx_drst_n   =1'b1;
reg         hsrx_en     =1'b0;
reg  [5:0]  hsrx_cnt    = 'b0;
wire        from0to3    = (lp_reg[1]==0)&(lp_reg[0]==3);
wire        from1to0    = (lp_reg[1]==1)&(lp_reg[0]==0);
wire        from3to1    = (lp_reg[1]==3)&(lp_reg[0]==1);
wire        fromxto3    = (lp_reg[1]!=3)&(lp_reg[0]==3);
wire        odt_en     = odt_en_msk & ~(|LP_DATA0);

always @(posedge byte_clk or negedge RESET_N) begin
    if (~RESET_N)   lp_reg[0]   <= 2'b00;
    else if (1)     lp_reg[0]   <= LP_DATA0;

    if (~RESET_N)   lp_reg[1]   <= 2'b00;
    else if (1)     lp_reg[1]   <= lp_reg[0];

    if (~RESET_N)       odt_en_msk  <= 'b0;
    else if (from3to1)  odt_en_msk  <= 1'b1;
    else if (fromxto3)  odt_en_msk  <= 1'b0;

    if (~RESET_N)   rx_drst_n   <= 1'b1;
    else if (1)     rx_drst_n   <= from3to1 ? 1'b0
//    else if (1)     rx_drst_n   <= from0to3 | from3to1 ? 1'b0
                                 : from1to0 ? 1'b1 : rx_drst_n;

    if (~RESET_N)       hsrx_cnt    <= 'd0;
    else if (odt_en)    hsrx_cnt    <= `HS_EN_DLY;
    else if (hsrx_cnt>0)hsrx_cnt    <= hsrx_cnt - 2'd1;

    if (~RESET_N)   hsrx_en     <= 1'b0;
    else if (1)     hsrx_en     <= (hsrx_cnt>0);
end

MIPI_RX_PHY u_mipi_dphy
(   .reset_n        (RESET_N        )
,   .HS_CLK_P       (HS_CLK_P       )
,   .HS_CLK_N       (HS_CLK_N       )
,   .HS_DATA3_P     (HS_DATA3_P     )
,   .HS_DATA3_N     (HS_DATA3_N     )
,   .HS_DATA2_P     (HS_DATA2_P     )
,   .HS_DATA2_N     (HS_DATA2_N     )
,   .HS_DATA1_P     (HS_DATA1_P     )
,   .HS_DATA1_N     (HS_DATA1_N     )
,   .HS_DATA0_P     (HS_DATA0_P     )
,   .HS_DATA0_N     (HS_DATA0_N     )
//,   .hs_en          (hs_en          )
,   .hs_en          (hsrx_en        )
,   .clk_term_en    (1'b1           )
,   .data_term_en   (odt_en         )
,   .data_out3      (byte_d3        )
,   .data_out2      (byte_d2        )
,   .data_out1      (byte_d1        )
,   .data_out0      (byte_d0        )
,   .data_bf_lane3  (dbf_d3         )
,   .data_bf_lane2  (dbf_d2         )
,   .data_bf_lane1  (dbf_d1         )
,   .data_bf_lane0  (dbf_d0         )
,   .clk_byte_out   (byte_clk       )
,   .ready          (byte_ready     )
);

//MIPI_RX_PHY_CTRL #
//(   .HS_ENTER_DELAY (16             )
//,   .TERM_OFF_DELAY (12             )
//) u_dphy_ctrl
//(   .I_RSTN         (RESET_N        )
//,   .I_BYTE_CLK     (byte_clk       )
//,   .I_LP_D0        (LP_DATA0       )
//,   .O_HS_EN        (hs_en          )
//,   .O_TERM_ON      (term_en        )
//);


Dsi_Rx_Top u_dsi_rx
(   .I_RSTN         (RESET_N        )
,   .I_BYTE_CLK     (byte_clk       )
,   .I_REF_DT       (DT_RGB888      )
,   .I_READY        (byte_ready     )
,   .I_DATA0        (byte_d0        )
,   .I_DATA1        (byte_d1        )
,   .I_DATA2        (byte_d2        )
,   .I_DATA3        (byte_d3        )
,   .O_SP_EN        (sp_en          )
,   .O_LP_EN        (lp_en          )
,   .O_LP_AV_EN     (lp_av_en       )
,   .O_ECC_OK       (ecc_ok         )
,   .O_ECC          ()
,   .O_WC           (wc             )
,   .O_VC           ()
,   .O_DT           (dt             )
,   .O_PAYLOAD      (payload        )
,   .O_PAYLOAD_DV   (payload_dv     )
);

`ifdef USE_OSC
//! 105 * 4 / 3 = 140MHz
Gowin_rPLL_m4d3 u_pll_1
(   .clkout         (pixel_clk      )
,   .lock           (               )
,   .reset          (!RESET_N       )
,   .clkin          (byte_clk       )
);

//! 50 * 49 / 5 = 490 MHz
Gowin_rPLL_m49d5 u_pll_2
(   .clkout         (lvds_eclk      )
,   .reset          (!RESET_N       )
,   .clkin          (OSC_50M        )
,   .lock           (               )
);
//! 490 / 3.5 = 140 MHz
CLKDIV #
(   .DIV_MODE       ( "3.5"         )
) u_clk_div_3p5
(   .HCLKIN         (lvds_eclk      )
,   .RESETN         (RESET_N        )
,   .CALIB          (1'b0           )
,   .CLKOUT         (lvds_pclk      )
);

`else
//! 105 * 14 / 3 = 490 MHz
Gowin_rPLL_m14d3 u_pll
(   .clkout         (lvds_eclk      )
,   .reset          (!RESET_N       )
,   .clkin          (byte_clk       )
,   .lock           (               )
);
//! 490 / 3.5 = 140 MHz; 
CLKDIV #
(   .DIV_MODE       ("3.5"          )
) u_clk_div_3p5
(   .HCLKIN         (lvds_eclk      )
,   .RESETN         (RESET_N        )
,   .CALIB          (1'b0           )
,   .CLKOUT         (lvds_pclk      )
);
assign pixel_clk = lvds_pclk;
`endif

wire    w_sp_en = sp_en&ecc_ok;
wire    w_lp_av_en = lp_av_en&ecc_ok;

Byte_to_Pixel_Top u_b2p
(   .I_RSTN         (RESET_N        )
,   .I_BYTE_CLK     (byte_clk       )
,   .I_PIXEL_CLK    (pixel_clk      )
,   .I_SP_EN        (w_sp_en        )
,   .I_LP_AV_EN     (w_lp_av_en     )
,   .I_WC           (wc             )
,   .I_DT           (dt             )
,   .I_PAYLOAD_DV   (payload_dv     )
,   .I_PAYLOAD      (payload        )
,   .O_VSYNC        (b2p_vsync      )
,   .O_HSYNC        (b2p_hsync      )
,   .O_DE           (b2p_de         )
,   .O_PIXEL        (b2p_pixel      )
);

`ifdef USE_OSC
DPI_EXTRACTOR #
(   .InPolarity     (3'b111         )
,   .OutPolarity    (3'b111         )
,   .ExtrPattern    (`PATTERN       )
,   .ExtrPatternLen (`PATTERN_LEN   )
,   .ExtrStartX     (`START_X       )
,   .OutputWidth    (`WIDTH         )
,   .OutputHFP      (`HFP           )
,   .OutputHSW      (`HSW           )
,   .OutputHBP      (`HBP           )
,   .OutputHeight   (`HEIGHT        )
,   .OutputVFP      (`VFP           )
,   .OutputVSW      (`VSW           )
,   .OutputVBP      (`VBP           )
,   .VSyncDelay     (`VSYNC_DELAY   )
) u_extr
(   .I_RSTN         (RESET_N        )
,   .I_CLK_IN       (pixel_clk      )
,   .I_VS_IN        (b2p_vsync      )
,   .I_HS_IN        (b2p_hsync      )
,   .I_DE_IN        (b2p_de         )
,   .I_RGB_IN       (b2p_pixel      )
,   .I_CLK_OUT      (lvds_pclk      )
,   .O_VS           (ext_vsync      )
,   .O_HS           (ext_hsync      )
,   .O_DE           (ext_de         )
,   .O_RGB          (ext_pixel      )
);
assign lvds_r   = ext_pixel[ 7:0 ];
assign lvds_g   = ext_pixel[15:8 ];
assign lvds_b   = ext_pixel[23:16];
assign lvds_vs  = ext_vsync;
assign lvds_hs  = ext_hsync;
assign lvds_de  = ext_de;
`else
assign lvds_r   = b2p_pixel[ 7:0 ];
assign lvds_g   = b2p_pixel[15:8 ];
assign lvds_b   = b2p_pixel[23:16];
assign lvds_vs  = b2p_vsync;
assign lvds_hs  = b2p_hsync;
assign lvds_de  = b2p_de;
`endif

`ifdef JEIDA_MAPPING
wire [6:0] lvds_tx_0    = { lvds_r[2]   , lvds_r[3] , lvds_r[4] , lvds_r[5] , lvds_r[6] , lvds_r[7] , lvds_g[2]};
wire [6:0] lvds_tx_1    = { lvds_g[3]   , lvds_g[4] , lvds_g[5] , lvds_g[6] , lvds_g[7] , lvds_b[2] , lvds_b[3]};
wire [6:0] lvds_tx_2    = { lvds_b[4]   , lvds_b[5] , lvds_b[6] , lvds_b[7] , lvds_hs   , lvds_vs   , lvds_de  };
wire [6:0] lvds_tx_3    = { lvds_r[0]   , lvds_r[1] , lvds_g[0] , lvds_g[1] , lvds_b[0] , lvds_b[1] , 1'b0     };
`else
wire [6:0] lvds_tx_0    = { lvds_r[0]   , lvds_r[1] , lvds_r[2] , lvds_r[3] , lvds_r[4] , lvds_r[5] , lvds_g[0]};
wire [6:0] lvds_tx_1    = { lvds_g[1]   , lvds_g[2] , lvds_g[3] , lvds_g[4] , lvds_g[5] , lvds_b[0] , lvds_b[1]};
wire [6:0] lvds_tx_2    = { lvds_b[2]   , lvds_b[3] , lvds_b[4] , lvds_b[5] , lvds_hs   , lvds_vs   , lvds_de  };
wire [6:0] lvds_tx_3    = { lvds_r[6]   , lvds_r[7] , lvds_g[6] , lvds_g[7] , lvds_b[6] , lvds_b[7] , 1'b0     };
`endif

LVDS_7_to_1_TX u_lvds_tx_left
(   .RST_Tx             (!RESET_N       )
,   .eclk               (lvds_eclk      )     //PLL
,   .sync_clk           (lvds_pclk      )
,   .stop               (1'b0           )
,   .T0_in              (lvds_tx_0      )
,   .T1_in              (lvds_tx_1      )
,   .T2_in              (lvds_tx_2      )
,   .T3_in              (lvds_tx_3      )
//! output 
,   .TCLK_out_p         (LVDS_CLK_P     )
,   .TCLK_out_n         (LVDS_CLK_N     )
,   .T0_out_p           (LVDS_D0_P      )
,   .T0_out_n           (LVDS_D0_N      )
,   .T1_out_p           (LVDS_D1_P      )
,   .T1_out_n           (LVDS_D1_N      )
,   .T2_out_p           (LVDS_D2_P      )
,   .T2_out_n           (LVDS_D2_N      )
,   .T3_out_p           (LVDS_D3_P      )
,   .T3_out_n           (LVDS_D3_N      )
);

endmodule