`timescale 1ps/1ps

`define  MIPI_CLK_PERIOD    2350
`define  CLK_WAIT           1175
`define  MIPI_DELAY         235

`define  VH                 1080
`define  VSW                4
`define  VFP                8
`define  VBP                4

`define  VW                 1920
`define  HSW                40
`define  HFP                129
`define  HBP                40

`define  WC                 (`VW*3/4)

`define  PHDR               32'h3B16_803E

module tb_dsi2lvds();

reg [23:0]  pixel_data  [`VW-1:0];
reg [31:0]  byte_data   [`WC-1:0];
    
reg rst_n;
reg clk_50m;
reg mipi_clk_p;
reg mipi_clk_n;
reg [7:0] mipi_hsd;
reg [1:0] mipi_lpd;

Dsi2Lvds_Top u_dut
(   .RESET_N    ( rst_n         )
,   .OSC_50M    ( clk_50m       )
,   .HS_CLK_P   ( mipi_clk_p    )
,   .HS_CLK_N   ( mipi_clk_n    )
,   .HS_DATA3_P ( mipi_hsd[7]   )
,   .HS_DATA3_N ( mipi_hsd[6]   )
,   .HS_DATA2_P ( mipi_hsd[5]   )
,   .HS_DATA2_N ( mipi_hsd[4]   )
,   .HS_DATA1_P ( mipi_hsd[3]   )
,   .HS_DATA1_N ( mipi_hsd[2]   )
,   .HS_DATA0_P ( mipi_hsd[1]   )
,   .HS_DATA0_N ( mipi_hsd[0]   )
,   .LP_DATA0   ( mipi_lpd      )
`ifndef DEBUG
,   .LVDS_CLK_P ()
,   .LVDS_CLK_N ()
,   .LVDS_D0_P  ()
,   .LVDS_D0_N  ()
,   .LVDS_D1_P  ()
,   .LVDS_D1_N  ()
,   .LVDS_D2_P  ()
,   .LVDS_D2_N  ()
,   .LVDS_D3_P  ()
,   .LVDS_D3_N  ()
`endif
// ,   .LP_DATA0   ()
// ,   .led        ()
// ,   .dbg        ()
);


always begin
    #10000 clk_50m <= 0;
    #10000 clk_50m <= 1;
end

always begin
    #`CLK_WAIT mipi_clk_p <= 0; mipi_clk_n <= 1;
    #`CLK_WAIT mipi_clk_p <= 1; mipi_clk_n <= 0;
end

initial begin
    rst_n = 0;
    mipi_hsd = 8'h55;
    mipi_lpd = 3;
    #102000;
    rst_n = 1;

    repeat (100) @(posedge u_dut.pixel_clk);
    repeat (200) @(posedge u_dut.byte_clk);

    // SendVSS();
    // repeat (10) @(posedge u_dut.byte_clk);
    // SendHSE();
    // repeat (2000) @(posedge u_dut.byte_clk);

    // SendHSS();
    // repeat (10) @(posedge u_dut.byte_clk);
    // SendHSE();
    // repeat (2000) @(posedge u_dut.byte_clk);

    // SendVSE();
    // repeat (10) @(posedge u_dut.byte_clk);
    // SendHSE();
    // repeat (2000) @(posedge u_dut.byte_clk);

    SendPic();

    #10000000;
    $finish;
end

task PrepareLine;
    input [11:0] line;
    integer     i;
begin
    for (i=0; i<`VW; i=i+1) 
        pixel_data[i] = {line[11:0], i[19:0]};

    for (i=0; i<`VW/4; i=i+1) begin
        byte_data[i*3+2] = {pixel_data[i*4+3][23: 0], pixel_data[i*4+2][23:16]};
        byte_data[i*3+1] = {pixel_data[i*4+2][15: 0], pixel_data[i*4+1][23: 8]};
        byte_data[i*3+0] = {pixel_data[i*4+1][ 7: 0], pixel_data[i*4+0][23: 0]};
    end
end
endtask

task SetLP;
    input [1:0]  data;
begin
    @(edge mipi_clk_p); #`MIPI_DELAY;
    mipi_lpd = data;
end
endtask

task SetHS;
    input [3:0]  data;
begin
    @(edge mipi_clk_p); #`MIPI_DELAY;
    mipi_hsd = { data[3], ~data[3]
               , data[2], ~data[2]
               , data[1], ~data[1]
               , data[0], ~data[0]};
end
endtask

task SendWord;
    input [31:0] data;
    integer i;
begin
    for (i=0; i<8; i=i+1)
        SetHS({ data[24+i], data[16+i], data[8+i], data[i]});
    // end
end
endtask

task SendVSS;
begin
    SetLP(1);
    repeat (6) @(posedge u_dut.byte_clk);
    SetLP(0);
    SetHS(0);
    repeat (16) @(posedge u_dut.byte_clk);

    SendWord(32'hB8B8_B8B8);
    SendWord(32'h0700_0001);    // VSS
    SendWord(32'hFFFF_FFFF);
    repeat (16) @(posedge u_dut.byte_clk);
    SetLP(3);
    SetHS(0);

    repeat (12) @(posedge u_dut.byte_clk);
end
endtask

task SendVSE;
begin
    SetLP(1);
    repeat (6) @(posedge u_dut.byte_clk);
    SetLP(0);
    SetHS(0);
    repeat (16) @(posedge u_dut.byte_clk);

    SendWord(32'hB8B8_B8B8);
    SendWord(32'h1400_0011);    // VSE
    SendWord(32'hFFFF_FFFF);
    repeat (16) @(posedge u_dut.byte_clk);
    SetLP(3);
    SetHS(0);

    repeat (12) @(posedge u_dut.byte_clk);
end
endtask

task SendHSS;
begin
    SetLP(1);
    repeat (6) @(posedge u_dut.byte_clk);
    SetLP(0);
    SetHS(0);
    repeat (16) @(posedge u_dut.byte_clk);

    SendWord(32'hB8B8_B8B8);
    SendWord(32'h1200_0021);    // HSS
    SendWord(32'hFFFF_FFFF);
    repeat (16) @(posedge u_dut.byte_clk);
    SetLP(3);
    SetHS(0);

    repeat (12) @(posedge u_dut.byte_clk);
end
endtask

task SendHSE;
begin
    SetLP(1);
    repeat (6) @(posedge u_dut.byte_clk);
    SetLP(0);
    SetHS(0);
    repeat (16) @(posedge u_dut.byte_clk);

    SendWord(32'hB8B8_B8B8);
    SendWord(32'h0100_0031);    // HSE
    SendWord(32'hFFFF_FFFF);
    repeat (16) @(posedge u_dut.byte_clk);
    SetLP(3);
    SetHS(0);

    repeat (12) @(posedge u_dut.byte_clk);
end
endtask

task SendLine;
    integer i;
begin
    SetLP(1);
    repeat (6) @(posedge u_dut.byte_clk);
    SetLP(0);
    SetHS(0);
    repeat (16) @(posedge u_dut.byte_clk);

    SendWord(32'hB8B8_B8B8);
    SendWord(`PHDR);
    for (i=0; i<`WC; i=i+1)
        SendWord(byte_data[i]);
    SendWord(32'h0000_AA55);
    SendWord(32'hFFFF_00FF);
    repeat (16) @(posedge u_dut.byte_clk);
    SetLP(3);
    SetHS(0);

    repeat (12) @(posedge u_dut.byte_clk);
end
endtask


localparam VsyEnd = `VSW;
localparam VbpEnd = `VSW+`VBP;
localparam VacEnd = `VSW+`VBP+`VH;
localparam AllEnd = `VSW+`VBP+`VH+`VFP;

task SendPic;
    integer     i;
begin
    for (i=0; i<AllEnd; i=i+1) begin
        if (i==0) begin
            SendVSS();
            repeat (4) @(posedge u_dut.byte_clk);
            SendHSE();
            repeat (1600) @(posedge u_dut.byte_clk);
        end else if (i==VsyEnd) begin
            SendVSE();
            repeat (4) @(posedge u_dut.byte_clk);
            SendHSE();
            repeat (1600) @(posedge u_dut.byte_clk);
        end else if (i<VacEnd && i>=VbpEnd) begin
            PrepareLine(i-VbpEnd);
            SendHSS();
            repeat (4) @(posedge u_dut.byte_clk);
            SendHSE();
            repeat (4) @(posedge u_dut.byte_clk);
            SendLine();
        end else begin
            SendHSS();
            repeat (4) @(posedge u_dut.byte_clk);
            SendHSE();
            repeat (1600) @(posedge u_dut.byte_clk);
        end
    end
end
endtask
    

endmodule