
`define     JEIDA_MAPPING
`define     USE_OSC

// `define     DEBUG

`ifdef USE_OSC
//! MIPI: VFP/VSW/VBP = 8/4/4
//! PCLK : 490 / 3.5 = 140 MHZ
//! (129+40+40+1920) * (1080+8+4+4) * 60 ~= 140 MHz
// `define     PATTERN_LEN     2
// `define     PATTERN         2'b11
// `define     START_X         0
// `define     WIDTH           1920
// `define     HEIGHT          1080
// `define     VFP             8
// `define     VSW             4
// `define     VBP             4
// `define     HFP             129
// `define     HSW             40
// `define     HBP             40
// `define     VSYNC_DELAY     180

//! MIPI: VFP/VSW/VBP = 16/6/6
//! PCLK : 475 / 3.5 = 135.714 MHZ
//! (1920+62+30+30) *(1080+16+6+6) * 60 ~= 135.752M
`ifndef SIM
`define     PATTERN_LEN     2
`define     PATTERN         2'b11
`define     START_X         0
`define     WIDTH           1920
`define     HEIGHT          1080
`define     VFP             16
`define     VSW             6
`define     VBP             6
`define     HFP             63
`define     HSW             30
`define     HBP             30
`define     VSYNC_DELAY     180
`else
`define     PATTERN_LEN     2
`define     PATTERN         2'b11
`define     START_X         0
`define     WIDTH           1920
`define     HEIGHT          1080
`define     VFP             8
`define     VSW             4
`define     VBP             4
`define     HFP             129
`define     HSW             40
`define     HBP             40
`define     VSYNC_DELAY     180
`endif
`endif

`define     HS_EN_DLY       'd10

module Dsi2Lvds_Top
(   input           RESET_N
`ifdef USE_OSC
,   input           OSC_50M
`endif
//! MIPI Port: fixed; NONE
,   inout           ck_n 
,   inout           ck_p 
,   inout           d0_n
,   inout           d0_p
,   inout           d1_n
,   inout           d1_p
,   inout           d2_n
,   inout           d2_p
,   inout           d3_n
,   inout           d3_p
//! output
`ifndef DEBUG
,   output          LVDS_CLK_P
,   output          LVDS_CLK_N
,   output          LVDS_D0_P
,   output          LVDS_D0_N
,   output          LVDS_D1_P
,   output          LVDS_D1_N
,   output          LVDS_D2_P
,   output          LVDS_D2_N
,   output          LVDS_D3_P
,   output          LVDS_D3_N
`endif
,   output [2:0]    led
);
localparam  DT_RGB888 = 6'h3E;

//! byte_clk = bit_clk / 4 = 420 / 4 = 105MHz
wire            byte_clk    /* synthesis syn_keep = 1 */;
reg  [1:0]      lp_reg      [1:0];
reg             odt_en_msk  = 'b0;
reg             rx_drst_n   =1'b1;
reg             hsrx_en_msk =1'b0;
reg  [5:0]      hsrx_cnt    = 'b0;
//

wire [3:0]      hsrxd_vld   ;
wire [15:0]     d0ln_hsrxd  ;
wire [15:0]     d1ln_hsrxd  ;
wire [15:0]     d2ln_hsrxd  ;
wire [15:0]     d3ln_hsrxd  ;

reg             byte_ready  ;
reg  [7:0]      byte_d0     ;
reg  [7:0]      byte_d1     ;
reg  [7:0]      byte_d2     ;
reg  [7:0]      byte_d3     ;

// wire [7:0]      dbf_d3 , dbf_d2 , dbf_d1 , dbf_d0 ;

wire            pixel_clk   /* synthesis syn_keep = 1 */;
wire            lvds_eclk   /* synthesis syn_keep = 1 */;
wire            lvds_pclk   /* synthesis syn_keep = 1 */;
wire            sp_en       /* synthesis syn_keep = 1 */;
wire            lp_en       /* synthesis syn_keep = 1 */;
wire            lp_av_en    /* synthesis syn_keep = 1 */;
wire            ecc_ok      ;
wire [15:0]     wc          ;
wire [5:0]      dt          ;
wire [3:0]      payload_dv  /* synthesis syn_keep = 1 */;
wire [31:0]     payload     /* synthesis syn_keep = 1 */;

wire            b2p_vsync   ;
wire            b2p_hsync   ;
wire            b2p_de      ;
wire [23:0]     b2p_pixel   ;

wire            ext_vsync   ;
wire            ext_hsync   ;
wire            ext_de      ;
wire [23:0]     ext_pixel   ;

reg             vsync       ;
reg  [4:0]      frame       ;
reg             frame_led   ;
wire            vs_pulse    ;

wire [ 7:0]     lvds_r      ;
wire [ 7:0]     lvds_g      ;
wire [ 7:0]     lvds_b      ;
wire            lvds_vs     ;
wire            lvds_hs     ;
wire            lvds_de     ;
//
reg  [31:0]     run_cnt     ;
wire            run_led     ;
wire  [1:0]     lp_data0;
//
wire            from0to3    = (lp_reg[1]==0)&(lp_reg[0]==3);
wire            from1to0    = (lp_reg[1]==1)&(lp_reg[0]==0);
wire            from3to1    = (lp_reg[1]==3)&(lp_reg[0]==1);
wire            fromxto3    = (lp_reg[1]!=3)&(lp_reg[0]==3);
wire            odt_en      = odt_en_msk & (lp_reg[0]==0);

`ifdef SIM
wire VCC;
VCC VCC_cZ (
    .V(VCC)
);

GSR GSR (
    .GSRI(VCC) 
);
`endif


always @(posedge byte_clk or negedge RESET_N) begin
    if (~RESET_N)   lp_reg[0]   <= 2'b00;
    else if (1)     lp_reg[0]   <= lp_data0;

    if (~RESET_N)   lp_reg[1]   <= 2'b00;
    else if (1)     lp_reg[1]   <= lp_reg[0];

    if (~RESET_N)       odt_en_msk  <= 'b0;
    else if (from3to1)  odt_en_msk  <= 1'b1;
    else if (fromxto3)  odt_en_msk  <= 1'b0;

    if (~RESET_N)   rx_drst_n   <= 1'b1;
    else if (1)     rx_drst_n   <= from3to1 ? 1'b0
//    else if (1)     rx_drst_n   <= from0to3 | from3to1 ? 1'b0
                                 : from1to0 ? 1'b1 : rx_drst_n;

    if (~RESET_N)       hsrx_cnt    <= 'd0;
    else if (odt_en)    hsrx_cnt    <= `HS_EN_DLY;
    else if (hsrx_cnt>0)hsrx_cnt    <= hsrx_cnt - 2'd1;

    if (~RESET_N)       hsrx_en_msk <= 1'b0;
    else if (1)         hsrx_en_msk <= (hsrx_cnt>0);
end


Gowin_MIPI_DPHY_RX u_dphy_rx
(   .ck_n               ( ck_n          ) //! inout ck_n, physical interface, Do not connect
,   .ck_p               ( ck_p          ) //! inout ck_p, physical interface, Do not connect
,   .rx0_n              ( d0_n          ) //! inout rx0_n, physical interface, Do not connect
,   .rx0_p              ( d0_p          ) //! inout rx0_p, physical interface, Do not connect
,   .rx1_n              ( d1_n          ) //! inout rx1_n, physical interface, Do not connect
,   .rx1_p              ( d1_p          ) //! inout rx1_p, physical interface, Do not connect
,   .rx2_n              ( d2_n          ) //! inout rx2_n, physical interface, Do not connect
,   .rx2_p              ( d2_p          ) //! inout rx2_p, physical interface, Do not connect
,   .rx3_n              ( d3_n          ) //! inout rx3_n, physical interface, Do not connect
,   .rx3_p              ( d3_p          ) //! inout rx3_p, physical interface, Do not connect
    
,   .rx_clk_o           ( byte_clk      ) //output HSRX Clock output
,   .rx_clk_1x          ( byte_clk      ) //input 1X clock from fabric, max 93.75MHz@1.5Gbps

,   .drst_n             ( rx_drst_n     ) //input digital reset, active low
,   .pwron              (1'b1           ) //input PowerOn Control: 1'b1: HSRX on; 1'b0: HSRX off to standby in low power state
,   .reset              (~RESET_N       ) //input Reset signal, 1'b1: reset all; 
,   .hsrx_stop          (1'b0           ) //input HSRX Clock Stop Signal for synchronization

,   .hs_8bit_mode       (1'b1           ) //input 1'b1:8bit output mode; 1'b0:16bit output mode
,   .rx_invert          (1'b0           ) //input data polarity selection
,   .byte_lendian       (1'b1           ) //input 1'b1: Littlendian
,   .lalign_en          (1'b1           ) //input lane aligner enable

,   .walign_by          (1'b0           ) //input word aligner bypass   =====> bypass word align for PRBS test
,   .one_byte0_match    (1'b0           ) //input byte count match in word aligner: 1'b0:2bytes, 1'b1:3bytes
,   .word_lendian       (1'b1           ) //input data little/big endian of dual word(16bit, 8bit/word). Not used in 8bit data output mode
,   .fifo_rd_std        (3'b001         ) //input FIFO read threadhold; Can only be 1 in 8bit mode.
,   .walign_dvld        (1'b0           ) //word aligner input data valid from the fabric

,   .hsrx_en_ck         (1'b1           ) //input CK Lane: 1'b1 HSRX enabled
,   .d0ln_hsrx_dren     (1'b1           ) //input Data Lane HSRX driver enabled
,   .d1ln_hsrx_dren     (1'b1           ) //input Data Lane HSRX driver enabled
,   .d2ln_hsrx_dren     (1'b1           ) //input Data Lane HSRX driver enabled
,   .d3ln_hsrx_dren     (1'b1           ) //input Data Lane HSRX driver enabled
,   .hsrx_odten_ck      (1'b1           ) //input CK Lane HSRX ODT enabled
,   .hsrx_odten_d0      ( odt_en       ) //input D0 Lane HSRX ODT enabled
,   .hsrx_odten_d1      ( odt_en       ) //input D1 Lane HSRX ODT enabled
,   .hsrx_odten_d2      ( odt_en       ) //input D2 Lane HSRX ODT enabled
,   .hsrx_odten_d3      ( odt_en       ) //input D3 Lane HSRX ODT enabled
,   .d0ln_hsrxd_vld     ( hsrxd_vld[0]  ) //output d0ln_hsrxd_vld
,   .d1ln_hsrxd_vld     ( hsrxd_vld[1]  ) //output d1ln_hsrxd_vld
,   .d2ln_hsrxd_vld     ( hsrxd_vld[2]  ) //output d2ln_hsrxd_vld
,   .d3ln_hsrxd_vld     ( hsrxd_vld[3]  ) //output d3ln_hsrxd_vld
,   .d0ln_hsrxd         ( d0ln_hsrxd    ) //output [15:0] d0ln_hsrxd
,   .d1ln_hsrxd         ( d1ln_hsrxd    ) //output [15:0] d1ln_hsrxd
,   .d2ln_hsrxd         ( d2ln_hsrxd    ) //output [15:0] d2ln_hsrxd
,   .d3ln_hsrxd         ( d3ln_hsrxd    ) //output [15:0] d3ln_hsrxd


,   .lprx_en_ck         (1'b1           ) //input lprx_en_ck
,   .lprx_en_d0         (1'b1           ) //input lprx_en_d0
,   .lprx_en_d1         (1'b1           ) //input lprx_en_d1
,   .lprx_en_d2         (1'b1           ) //input lprx_en_d2
,   .lprx_en_d3         (1'b1           ) //input lprx_en_d3
,   .di_lprxck_n        (               ) //output di_lprxck_n
,   .di_lprxck_p        (               ) //output di_lprxck_p
,   .di_lprx0_n         ( lp_data0[0]   ) //output di_lprx0_n
,   .di_lprx0_p         ( lp_data0[1]   ) //output di_lprx0_p
,   .di_lprx1_n         (               ) //output di_lprx1_n
,   .di_lprx1_p         (               ) //output di_lprx1_p
,   .di_lprx2_n         (               ) //output di_lprx2_n
,   .di_lprx2_p         (               ) //output di_lprx2_p
,   .di_lprx3_n         (               ) //output di_lprx3_n
,   .di_lprx3_p         (               ) //output di_lprx3_p

,   .lptx_en_ck         (1'b0          ) //input lptx_en_ck
,   .lptx_en_d0         (1'b0          ) //input lptx_en_d0
,   .lptx_en_d1         (1'b0          ) //input lptx_en_d1
,   .lptx_en_d2         (1'b0          ) //input lptx_en_d2
,   .lptx_en_d3         (1'b0          ) //input lptx_en_d3
,   .do_lptxck_n        (1'b0          ) //input do_lptxck_n
,   .do_lptxck_p        (1'b0          ) //input do_lptxck_p
,   .do_lptx0_n         (1'b0          ) //input do_lptx0_n
,   .do_lptx0_p         (1'b0          ) //input do_lptx0_p
,   .do_lptx1_n         (1'b0          ) //input do_lptx1_n
,   .do_lptx1_p         (1'b0          ) //input do_lptx1_p
,   .do_lptx2_n         (1'b0          ) //input do_lptx2_n
,   .do_lptx2_p         (1'b0          ) //input do_lptx2_p
,   .do_lptx3_n         (1'b0          ) //input do_lptx3_n
,   .do_lptx3_p         (1'b0          ) //input do_lptx3_p

,   .deskew_by          (1'b1           ) //input deskew_by
,   .deskew_en_oedge    (1'b0           ) //input deskew_en_oedge
,   .deskew_req         (1'b0           ) //input deskew_req
,   .deskew_lnsel       ( 'b0           ) //input [2:0] selection of lane to config delay overwrite value and make lane in deskew delay overwrite mode
,   .deskew_lsb_mode    ( 'b0           ) //input [1:0] deskew_lsb_mode
,   .deskew_m           ( 'b0           ) //input [2:0] counter threathold for confirming edge
,   .deskew_mset        ( 'b0           ) //input [6:0] deskew_mset
,   .deskew_mth         ( 'b0           ) //input [12:0] deskew_mth
,   .deskew_owval       ( 'b0           ) //input [6:0] deskew_owval
,   .deskew_half_opening( 'b0           ) //input [5:0] deskew_half_opening
,   .deskew_oclkedg_en  (1'b0           ) //input deskew_oclkedg_en
,   .deskew_error       (               ) //output deskew_error
,   .d0ln_deskew_done   (               ) //output d0ln_deskew_done
,   .d1ln_deskew_done   (               ) //output d1ln_deskew_done
,   .d2ln_deskew_done   (               ) //output d2ln_deskew_done
,   .d3ln_deskew_done   (               ) //output d3ln_deskew_done

,   .eqcs_ck            (3'b100         ) //input [2:0] CK Lane: Equalizer Source degeneration capacitor setting, 3b'000 smallest Cap; 3b'111 bigest Cap, default 3'b100
,   .eqcs_lane0         (3'b100         ) //input [2:0] Data Lane0: Equalizer Source degeneration capacitor setting, 3b'000 smallest Cap; 3b'111 bigest Cap, default 3'b100
,   .eqcs_lane1         (3'b100         ) //input [2:0] Data Lane1: Equalizer Source degeneration capacitor setting, 3b'000 smallest Cap; 3b'111 bigest Cap, default 3'b100
,   .eqcs_lane2         (3'b100         ) //input [2:0] Data Lane2: Equalizer Source degeneration capacitor setting, 3b'000 smallest Cap; 3b'111 bigest Cap, default 3'b100
,   .eqcs_lane3         (3'b100         ) //input [2:0] Data Lane4: Equalizer Source degeneration capacitor setting, 3b'000 smallest Cap; 3b'111 bigest Cap, default 3'b100
,   .eqrs_ck            (3'b100         ) //input [2:0] CK Lane: Equalizer Source degeneration resistor setting, 3b'000 smallest res; 3b'111 bigest res, default 3'b100
,   .eqrs_lane0         (3'b100         ) //input [2:0] Data Lane0: Equalizer Source degeneration resistor setting, 3b'000 smallest res; 3b'111 bigest res, default 3'b100
,   .eqrs_lane1         (3'b100         ) //input [2:0] Data Lane1: Equalizer Source degeneration resistor setting, 3b'000 smallest res; 3b'111 bigest res, default 3'b100
,   .eqrs_lane2         (3'b100         ) //input [2:0] Data Lane2: Equalizer Source degeneration resistor setting, 3b'000 smallest res; 3b'111 bigest res, default 3'b100
,   .eqrs_lane3         (3'b100         ) //input [2:0] Data Lane4: Equalizer Source degeneration resistor setting, 3b'000 smallest res; 3b'111 bigest res, default 3'b100
,   .hsrx_dlydir_ck     (1'b0           ) //input CK Lane: Direction for HSRX Deskew Delay Control: 0 Count Up; 1 Count Down
,   .hsrx_dlydir_lane0  (1'b0           ) //input Data Lane0: Direction for HSRX Deskew Delay Control: 0 Count Up; 1 Count Down
,   .hsrx_dlydir_lane1  (1'b0           ) //input Data Lane1: Direction for HSRX Deskew Delay Control: 0 Count Up; 1 Count Down
,   .hsrx_dlydir_lane2  (1'b0           ) //input Data Lane2: Direction for HSRX Deskew Delay Control: 0 Count Up; 1 Count Down
,   .hsrx_dlydir_lane3  (1'b0           ) //input Data Lane4: Direction for HSRX Deskew Delay Control: 0 Count Up; 1 Count Down
,   .hsrx_dlyldn_ck     (1'b0           ) //input CK Lane: Load HSRX Deskew Delay Control input from Fuse: 1'b0 load;
,   .hsrx_dlyldn_lane0  (1'b0           ) //input Data Lane0: Load HSRX Deskew Delay Control input from Fuse: 1'b0 load;
,   .hsrx_dlyldn_lane1  (1'b0           ) //input Data Lane1: Load HSRX Deskew Delay Control input from Fuse: 1'b0 load;
,   .hsrx_dlyldn_lane2  (1'b0           ) //input Data Lane2: Load HSRX Deskew Delay Control input from Fuse: 1'b0 load;
,   .hsrx_dlyldn_lane3  (1'b0           ) //input Data Lane3: Load HSRX Deskew Delay Control input from Fuse: 1'b0 load;
,   .hsrx_dlymv_ck      (1'b0           ) //input CK Lane: enable HSRX Deskew Delay Control to count: 1'b1 move;
,   .hsrx_dlymv_lane0   (1'b0           ) //input Data Lane0: enable HSRX Deskew Delay Control to count: 1'b1 move;
,   .hsrx_dlymv_lane1   (1'b0           ) //input Data Lane1: enable HSRX Deskew Delay Control to count: 1'b1 move;
,   .hsrx_dlymv_lane2   (1'b0           ) //input Data Lane2: enable HSRX Deskew Delay Control to count: 1'b1 move;
,   .hsrx_dlymv_lane3   (1'b0           ) //input Data Lane3: enable HSRX Deskew Delay Control to count: 1'b1 move;
);


always @(posedge byte_clk or negedge RESET_N) begin
    if (~RESET_N)   byte_ready  <= 1'b0;
    else if (1)     byte_ready  <= hsrxd_vld[0] & hsrx_en_msk;

    if (~RESET_N)   byte_d0  <= 8'b0;
    else if (1)     byte_d0  <= d0ln_hsrxd[7:0];

    if (~RESET_N)   byte_d1  <= 8'b0;
    else if (1)     byte_d1  <= d1ln_hsrxd[7:0];

    if (~RESET_N)   byte_d2  <= 8'b0;
    else if (1)     byte_d2  <= d2ln_hsrxd[7:0];

    if (~RESET_N)   byte_d3  <= 8'b0;
    else if (1)     byte_d3  <= d3ln_hsrxd[7:0];
end

Dsi_Rx_Top u_dsi_rx
(   .I_RSTN         ( RESET_N       )
,   .I_BYTE_CLK     ( byte_clk      )
,   .I_REF_DT       ( DT_RGB888     )
,   .I_READY        ( byte_ready    )
,   .I_DATA0        ( byte_d0       )
,   .I_DATA1        ( byte_d1       )
,   .I_DATA2        ( byte_d2       )
,   .I_DATA3        ( byte_d3       )
,   .O_SP_EN        ( sp_en         )
,   .O_LP_EN        ( lp_en         )
,   .O_LP_AV_EN     ( lp_av_en      )
,   .O_ECC_OK       ( ecc_ok        )
,   .O_ECC          ()
,   .O_WC           ( wc            )
,   .O_VC           ()
,   .O_DT           ( dt            )
,   .O_PAYLOAD      ( payload       )
,   .O_PAYLOAD_DV   ( payload_dv    )
);

`ifdef USE_OSC
//! 105*28/6 = 490; 105*28/21 = 140 = 490*4/3
Gowin_PLL_in_105M u_pll_1
(   .reset      (~RESET_N   ) //input reset
,   .clkin      ( byte_clk  ) //input clkin
,   .lock       (           ) //output lock
,   .clkout0    (           ) //output clkout0, 490M
,   .clkout1    ( pixel_clk ) //output clkout1, 140M
);

//! 50*19/2 = 475M; 50*19/7=135.71M
Gowin_PLL_in_50M u_pll_50m
(   .reset      (~RESET_N   ) //input reset
,   .clkin      ( OSC_50M   ) //input clkin
,   .lock       (           ) //output lock
,   .clkout0    ( lvds_eclk ) //output clkout0, 475M
,   .clkout1    ( lvds_pclk ) //output clkout1, 135.71M
);

`else
//! 105*28/6 = 490; 105*28/21 = 140 = (105*4/3)
Gowin_PLL_in_105M u_pll_1
(   .reset      (~RESET_N   ) //input reset
,   .clkin      ( byte_clk  ) //input clkin
,   .lock       (           ) //output lock
,   .clkout0    ( lvds_eclk ) //output clkout0, 490M
,   .clkout1    ( lvds_pclk ) //output clkout1, 140M
);
assign  pixel_clk = lvds_pclk;
`endif

Byte_to_Pixel_Top u_b2p
(   .I_RSTN         (RESET_N        )
,   .I_BYTE_CLK     (byte_clk       )
,   .I_PIXEL_CLK    (pixel_clk      )
,   .I_SP_EN        (sp_en&ecc_ok   )
,   .I_LP_AV_EN     (lp_av_en&ecc_ok)
,   .I_WC           (wc             )
,   .I_DT           (dt             )
,   .I_PAYLOAD_DV   (payload_dv     )
,   .I_PAYLOAD      (payload        )
,   .O_VSYNC        (b2p_vsync      )
,   .O_HSYNC        (b2p_hsync      )
,   .O_DE           (b2p_de         )
,   .O_PIXEL        (b2p_pixel      )
);

`ifdef USE_OSC
DPI_EXTRACTOR #
(   .InPolarity     (3'b111         )
,   .OutPolarity    (3'b111         )
,   .ExtrPattern    (`PATTERN       )
,   .ExtrPatternLen (`PATTERN_LEN   )
,   .ExtrStartX     (`START_X       )
,   .OutputWidth    (`WIDTH         )
,   .OutputHFP      (`HFP           )
,   .OutputHSW      (`HSW           )
,   .OutputHBP      (`HBP           )
,   .OutputHeight   (`HEIGHT        )
,   .OutputVFP      (`VFP           )
,   .OutputVSW      (`VSW           )
,   .OutputVBP      (`VBP           )
,   .VSyncDelay     (`VSYNC_DELAY   )
) u_extr
(   .I_RSTN         (RESET_N        )
,   .I_CLK_IN       (pixel_clk      )
,   .I_VS_IN        (b2p_vsync      )
,   .I_HS_IN        (b2p_hsync      )
,   .I_DE_IN        (b2p_de         )
,   .I_RGB_IN       (b2p_pixel      )
,   .I_CLK_OUT      (lvds_pclk      )
,   .O_VS           (ext_vsync      )
,   .O_HS           (ext_hsync      )
,   .O_DE           (ext_de         )
,   .O_RGB          (ext_pixel      )
);

assign lvds_r   = ext_pixel[ 7:0 ];
assign lvds_g   = ext_pixel[15:8 ];
assign lvds_b   = ext_pixel[23:16];
assign lvds_vs  = ext_vsync;
assign lvds_hs  = ext_hsync;
assign lvds_de  = ext_de;
`else
assign lvds_r   = b2p_pixel[ 7:0 ];
assign lvds_g   = b2p_pixel[15:8 ];
assign lvds_b   = b2p_pixel[23:16];
assign lvds_vs  = b2p_vsync;
assign lvds_hs  = b2p_hsync;
assign lvds_de  = b2p_de;
`endif


always @(posedge lvds_pclk or negedge RESET_N)
if (~RESET_N)       vsync  <= 1'b0;
else                vsync  <= ext_vsync;

assign  vs_pulse    = ~ext_vsync & vsync;

always @(posedge lvds_pclk or negedge RESET_N)
if (~RESET_N)       frame   <= 'b0;
else if (vs_pulse)  frame   <= (frame == 5'd29) ? 5'd0 : frame + 1'd1;

always @(posedge lvds_pclk or negedge RESET_N)
if (~RESET_N)           frame_led   <= 1'b0;
else if (frame==5'd29 &vs_pulse)  frame_led   <= ~frame_led;



`ifdef JEIDA_MAPPING
wire [6:0] lvds_tx_0    = { lvds_r[2]   , lvds_r[3] , lvds_r[4] , lvds_r[5] , lvds_r[6] , lvds_r[7] , lvds_g[2]};
wire [6:0] lvds_tx_1    = { lvds_g[3]   , lvds_g[4] , lvds_g[5] , lvds_g[6] , lvds_g[7] , lvds_b[2] , lvds_b[3]};
wire [6:0] lvds_tx_2    = { lvds_b[4]   , lvds_b[5] , lvds_b[6] , lvds_b[7] , lvds_hs   , lvds_vs   , lvds_de  };
wire [6:0] lvds_tx_3    = { lvds_r[0]   , lvds_r[1] , lvds_g[0] , lvds_g[1] , lvds_b[0] , lvds_b[1] , 1'b0     };
`else
wire [6:0] lvds_tx_0    = { lvds_r[0]   , lvds_r[1] , lvds_r[2] , lvds_r[3] , lvds_r[4] , lvds_r[5] , lvds_g[0]};
wire [6:0] lvds_tx_1    = { lvds_g[1]   , lvds_g[2] , lvds_g[3] , lvds_g[4] , lvds_g[5] , lvds_b[0] , lvds_b[1]};
wire [6:0] lvds_tx_2    = { lvds_b[2]   , lvds_b[3] , lvds_b[4] , lvds_b[5] , lvds_hs   , lvds_vs   , lvds_de  };
wire [6:0] lvds_tx_3    = { lvds_r[6]   , lvds_r[7] , lvds_g[6] , lvds_g[7] , lvds_b[6] , lvds_b[7] , 1'b0     };
`endif

`ifndef DEBUG
LVDS_7_to_1_TX u_lvds_tx_left
(   .RST_Tx             (~RESET_N       )
,   .eclk               (lvds_eclk      )     //PLL
,   .sync_clk           (lvds_pclk      )
,   .stop               (1'b0           )
,   .T0_in              (lvds_tx_0      )
,   .T1_in              (lvds_tx_1      )
,   .T2_in              (lvds_tx_2      )
,   .T3_in              (lvds_tx_3      )
//! output 
,   .TCLK_out_p         (LVDS_CLK_P     )
,   .TCLK_out_n         (LVDS_CLK_N     )
,   .T0_out_p           (LVDS_D0_P      )
,   .T0_out_n           (LVDS_D0_N      )
,   .T1_out_p           (LVDS_D1_P      )
,   .T1_out_n           (LVDS_D1_N      )
,   .T2_out_p           (LVDS_D2_P      )
,   .T2_out_n           (LVDS_D2_N      )
,   .T3_out_p           (LVDS_D3_P      )
,   .T3_out_n           (LVDS_D3_N      )
);

assign  led[1] = frame_led;
assign  led[2] = frame_led;

`else

reg         lvds_vs_r;
reg         lvds_hs_r;
reg         lvds_de_r;
reg  [7:0]  lvds_r_r;
reg  [7:0]  lvds_g_r;
reg  [7:0]  lvds_b_r;


always @(posedge lvds_pclk) begin
    lvds_vs_r   <= lvds_vs;
    lvds_hs_r   <= lvds_hs;
    lvds_de_r   <= lvds_de;
    lvds_r_r    <= lvds_r;
    lvds_g_r    <= lvds_g;
    lvds_b_r    <= lvds_b;
end

assign  led[1]  = 1'b0
                | lvds_vs_r
                | lvds_hs_r
                | lvds_de_r
                | ^lvds_r_r
                | ^lvds_g_r
                | ^lvds_b_r
                ;
assign  led[2]  = byte_clk;

`endif

//=============================================================================
// running logic
always @(posedge lvds_pclk or negedge RESET_N)
if (~RESET_N)   run_cnt  <= 'b0;
else            run_cnt  <= run_cnt + 1'd1;

assign  run_led = run_cnt[25];

assign  led[0] = run_led  ;

endmodule