`timescale 1ps/1ps

`define  MIPI_CLK_PERIOD    1876
`define  CLK_WAIT           938
`define  MIPI_DELAY         187

`define  VH                 1080
`define  VSW                4
`define  VFP                8
`define  VBP                4

`define  VW                 1920
`define  HSW                40
`define  HFP                129
`define  HBP                40

`define  WC                 (`VW/2)

`define  PHDR_L             16'h802A
`define  PHDR_H             16'h0B07

module tb_csi_demo();

reg [ 7:0]  pixel_data  [`VW-1:0];
reg [15:0]  byte_data   [`WC-1:0];
    
reg         rst_n;
reg         clk_50m;
reg         mipi_clk;
reg [1:0]   mipi_lpd;
reg [3:0]   mipi_hsd;
reg         mipi_hen;

wire ck_n;
wire ck_p;
wire d0_n;
wire d0_p;
wire d1_n;
wire d1_p;
wire d2_n;
wire d2_p;
wire d3_n;
wire d3_p;

CSI_Demo_Top u_dut
(   .RESET_N        ( rst_n     )
,   .OSC_50M        ( clk_50m   )
,   .ck_n           ( ck_n      )
,   .ck_p           ( ck_p      )
,   .d0_n           ( d0_n      )
,   .d0_p           ( d0_p      )
,   .d1_n           ( d1_n      )
,   .d1_p           ( d1_p      )

,   .O_CSI_RESETB   ()
,   .O_CSI_PWDN     ()
,   .O_CSI_SCL      ()
,   .IO_CSI_SDA     ()

,   .led        ()
);

bufif1 (weak1,weak0/*strong0*/) b_ck_hp (ck_p, mipi_clk, 1'b1);
notif1 (weak1,weak0/*strong0*/) b_ck_hn (ck_n, mipi_clk, 1'b1);

bufif1 (weak1,weak0/*strong0*/) b_d1_hp (d1_p, mipi_hsd[3], mipi_hen);
notif1 (weak1,weak0/*strong0*/) b_d1_hn (d1_n, mipi_hsd[3], mipi_hen);
bufif1 (weak1,weak0/*strong0*/) b_d0_hp (d0_p, mipi_hsd[1], mipi_hen);
notif1 (weak1,weak0/*strong0*/) b_d0_hn (d0_n, mipi_hsd[1], mipi_hen);

bufif0 b_d1_lp (d1_p, mipi_lpd[1], mipi_hen);
bufif0 b_d1_ln (d1_n, mipi_lpd[0], mipi_hen);
bufif0 b_d0_lp (d0_p, mipi_lpd[1], mipi_hen);
bufif0 b_d0_ln (d0_n, mipi_lpd[0], mipi_hen);

always begin
    #10000 clk_50m <= 0;
    #10000 clk_50m <= 1;
end

always begin
    #`CLK_WAIT mipi_clk <= 0;
    #`CLK_WAIT mipi_clk <= 1;
end

initial begin
    rst_n = 0;
    // mipi_hsd = 8'h55;
    mipi_lpd = 3;
    mipi_hen = 0;
    mipi_hsd = 0;
    #102000;
    rst_n = 1;

    repeat (100) @(posedge u_dut.pixel_clk);
    repeat (200) @(posedge u_dut.byte_clk);

    // SendVSS();
    // repeat (10) @(posedge u_dut.byte_clk);
    // SendHSE();
    // repeat (2000) @(posedge u_dut.byte_clk);

    // SendHSS();
    // repeat (10) @(posedge u_dut.byte_clk);
    // SendHSE();
    // repeat (2000) @(posedge u_dut.byte_clk);

    // SendVSE();
    // repeat (10) @(posedge u_dut.byte_clk);
    // SendHSE();
    // repeat (2000) @(posedge u_dut.byte_clk);

    SendPic();

    #10000000;
    $finish;
end

task PrepareLine;
    input [11:0] line;
    integer     i;
begin
    for (i=0; i<`VW; i=i+1) 
        pixel_data[i] = i % 256;

    for (i=0; i<`VW/2; i=i+1) begin
        byte_data[i] = {pixel_data[2*i+1], pixel_data[2*i]};
    end
end
endtask

task SetLP;
    input [1:0]  data;
begin
    @(edge mipi_clk); #`MIPI_DELAY;
    mipi_lpd = data;
end
endtask

task SetHS;
    input [1:0]  data;
begin
    @(edge mipi_clk); #`MIPI_DELAY;
    mipi_hsd = { data[1], ~data[1]
               , data[0], ~data[0]};
end
endtask

task SendWord;
    input [15:0] data;
    integer i;
begin
    for (i=0; i<8; i=i+1)
        SetHS({ data[8+i], data[i]});
    // end
end
endtask

task SendFVS;
begin
    SetLP(1);
    repeat (6) @(posedge u_dut.byte_clk);
    SetLP(0);
    SetHS(0); mipi_hen = 1;
    repeat (16) @(posedge u_dut.byte_clk);

    SendWord(16'hB8B8);
    SendWord(16'h0040);    // FVS
    SendWord(16'h1600);    //
    SendWord(16'hFFFF);
    repeat (16) @(posedge u_dut.byte_clk);
    SetLP(3); mipi_hen = 0;
    SetHS(0);

    repeat (12) @(posedge u_dut.byte_clk);
end
endtask

task SendFVE;
begin
    SetLP(1);
    repeat (6) @(posedge u_dut.byte_clk);
    SetLP(0);
    SetHS(0); mipi_hen = 1;
    repeat (16) @(posedge u_dut.byte_clk);

    SendWord(16'hB8B8);
    SendWord(16'h0041);    // FVE
    SendWord(16'h1100);    //
    SendWord(16'hFFFF);
    repeat (16) @(posedge u_dut.byte_clk);
    SetLP(3); mipi_hen = 0;
    SetHS(0);

    repeat (12) @(posedge u_dut.byte_clk);
end
endtask


task SendLine;
    integer i;
begin
    SetLP(1);
    repeat (6) @(posedge u_dut.byte_clk);
    SetLP(0);
    SetHS(0); mipi_hen = 1;
    repeat (16) @(posedge u_dut.byte_clk);

    SendWord(16'hB8B8);
    SendWord(`PHDR_L);
    SendWord(`PHDR_H);
    for (i=0; i<`WC; i=i+1)
        SendWord(byte_data[i]);
    SendWord(16'hAA55);
    SendWord(16'h00FF);
    repeat (16) @(posedge u_dut.byte_clk);
    SetLP(3); mipi_hen = 0;
    SetHS(0);

    repeat (12) @(posedge u_dut.byte_clk);
end
endtask


localparam VsyStart = `VSW;
localparam VacStart = `VSW+`VBP;
localparam VacEnd   = `VSW+`VBP+`VH;
localparam VsyEnd   = `VSW+`VBP+`VH+`VFP;

task SendPic;
    integer     i;
begin
    for (i=0; i<VsyEnd+1; i=i+1) begin
        if (i==VsyStart) begin
            SendFVS();
            repeat (1160) @(posedge u_dut.byte_clk);
        end else if (i==VsyEnd) begin
            SendFVE();
            repeat (1160) @(posedge u_dut.byte_clk);
        end else if (i<VacEnd && i>=VacStart) begin
            PrepareLine(i-VacStart);
            SendLine();
            repeat (200) @(posedge u_dut.byte_clk);
        end else begin
            repeat (1200) @(posedge u_dut.byte_clk);
        end
    end
end
endtask
    

endmodule